@@ -21,6 +21,7 @@ package Sidef::Deparse::Perl {
             between     => ";\n",
             after       => ";\n",
             namespaces  => [],
+            opt         => {},
 
             assignment_ops => {
                                '=' => '=',
@@ -68,6 +69,21 @@ use $];
 
 HEADER
 
+        if (exists $opts{opt}{A}) {
+            my $accuracy = abs(int($opts{opt}{A}));
+            $opts{header} .= "local \$Math::BigFloat::accuracy = $accuracy;\n";
+        }
+
+        if (exists $opts{opt}{P}) {
+            my $precision = abs(int($opts{opt}{P}));
+            $opts{header} .= "local \$Math::BigFloat::precision = -$precision;\n";
+        }
+
+        if (exists $opts{opt}{M}) {
+            my $mode = lc($opts{opt}{M}) =~ s/\s+//rg;
+            $opts{header} .= "local \$Math::BigFloat::round_mode = '${mode}';\n";
+        }
+
         %addr    = ();
         %type    = ();
         %top_add = ();
@@ -129,7 +145,7 @@ HEADER
         # Make sure that code-points between 128 and 256
         # will be stored internally as UTF-8 strings.
         if ($str =~ /[\200-\400]/) {
-            return "do {state \$x = do {require Encode; Encode::decode_utf8(Encode::encode_utf8($d))}}";
+            return "do {require Encode; Encode::decode_utf8(Encode::encode_utf8($d))}";
         }
 
         $d;
@@ -225,10 +241,10 @@ HEADER
         push @{$self->{block_declarations}},
           [$self->{current_block} // -1, 'my(' . join(', ', map { $self->_dump_var($_) } @vars) . ')' . ';'];
 
-        # Return the variables on assignments
+        # Return the lvalue variables on assignments
         if (@code > 1 or exists($init_obj->{args})) {
             push @code, '(' . join(', ', map { $self->_dump_var($_) } @vars) . ')';
-            return ('do { ' . join(';', @code) . '}');
+            return 'CORE::sub : lvalue {' . join(';', @code) . '}->()';
         }
 
         # Return one var as a list
@@ -658,7 +674,7 @@ HEADER
         }
         elsif ($ref eq 'Sidef::Types::Block::BlockInit') {
             if ($addr{$refaddr}++) {
-                $code = 'Sidef::Types::Block::Block->new(code => __SUB__';
+                $code = q{Sidef::Types::Block::Block->new(code => __SUB__, type => 'block'};
 
                 if (exists($obj->{init_vars}) and @{$obj->{init_vars}{vars}}) {
                     $code .= ', ' . $self->_dump_var_attr(@{$obj->{init_vars}{vars}});
@@ -887,8 +903,8 @@ HEADER
         elsif ($ref eq 'Sidef::Types::Regex::Regex') {
             $code =
               $self->make_constant($ref, 'new', "Regex$refaddr",
-                                   $self->_dump_string("$obj->{regex}"),
-                                   $obj->{global} ? '"g"' : ());
+                                   $self->_dump_string("$obj->{raw}"),
+                                   $self->_dump_string($obj->{flags} . ($obj->{global} ? 'g' : '')));
         }
         elsif ($ref eq 'Sidef::Types::Block::If' or $ref eq 'Sidef::Types::Block::While') {
             ## ok
@@ -1034,7 +1050,7 @@ HEADER
         elsif ($ref eq 'Sidef::Time::Time') {
             $code = $ref . '->new';
         }
-        elsif ($ref eq 'Sidef::Sys::SIG') {
+        elsif ($ref eq 'Sidef::Sys::Sig') {
             $code = $self->make_constant($ref, 'new', "Sig$refaddr");
         }
         elsif ($ref eq 'Sidef::Types::Number::Complex') {
@@ -1162,56 +1178,51 @@ HEADER
             $code = qq{CORE::die "Unimplemented at " . } . $self->_dump_string($obj->{file}) . qq{. " line $obj->{line}\\n"};
         }
 
-        # Array indices
+        # Array and hash indices
         if (exists $expr->{ind}) {
             my $limit = $#{$expr->{ind}};
             foreach my $i (0 .. $limit) {
+
                 my $ind = $expr->{ind}[$i];
+                if (exists $ind->{array}) {
 
-                if (substr($code, -1) eq '@') {
-                    $code .= $self->_dump_unpacked_indices($ind);
-                }
-                elsif ($#{$ind} > 0) {
-                    $code = '@{' . $code . '}' . $self->_dump_indices($ind);
+                    my $pos = $ind->{array};
+
+                    if (substr($code, -1) eq '@') {
+                        $code .= $self->_dump_unpacked_indices($pos);
+                    }
+                    else {
+                        $code = '@{' . $code . '}' . $self->_dump_indices($pos);
+                    }
                 }
                 else {
-                    $code .= '->' . $self->_dump_indices($ind);
+
+                    my $key = $ind->{hash};
+
+                    if (substr($code, -1) eq '@') {
+                        $code .= $self->_dump_unpacked_lookups($key);
+                    }
+                    else {
+                        $code = '@{' . $code . '}' . $self->_dump_lookups($key);
+                    }
                 }
 
-                if ($i < $limit and $#{$ind} == 0) {
-                    $code = '(' . $code . ' //= Sidef::Types::Array::Array->new' . ')';
+                if ($i < $limit) {
+                    if ($expr->{ind}[$i + 1]{array}) {
+                        $code = '(' . $code . ' //= Sidef::Types::Array::Array->new' . ')';
+                    }
+                    else {
+                        $code = '(' . $code . ' //= Sidef::Types::Hash::Hash->new' . ')';
+                    }
                 }
             }
         }
 
-        # Hash lookup
-        if (exists $expr->{lookup}) {
-            my $limit = $#{$expr->{lookup}};
-            foreach my $i (0 .. $limit) {
-                my $key = $expr->{lookup}[$i];
-
-                if (substr($code, -1) eq '@') {
-                    $code .= $self->_dump_unpacked_lookups($key);
-                }
-                elsif ($#{$key} > 0) {
-                    $code = '@{' . $code . '}' . $self->_dump_lookups($key);
-                }
-                else {
-                    $code .= '->' . $self->_dump_lookups($key);
-                }
-
-                if ($i < $limit and $#{$key} == 0) {
-                    $code = '(' . $code . ' //= Sidef::Types::Hash::Hash->new' . ')';
-                }
-            }
-        }
-
-        my $old_code = $code;
-
         # Method call on the self obj (+optional arguments)
         if (exists $expr->{call}) {
 
-            foreach my $i (0 .. $#{$expr->{call}}) {
+            my $end = $#{$expr->{call}};
+            foreach my $i (0 .. $end) {
 
                 my $call   = $expr->{call}[$i];
                 my $method = $call->{method};
@@ -1290,16 +1301,25 @@ HEADER
 
                     # Variable assignment (=)
                     if (exists($self->{assignment_ops}{$method})) {
-                        $code = "($code$self->{assignment_ops}{$method}" . $self->deparse_args(@{$call->{arg}}) . ")[0]";
+                        $code = "($code$self->{assignment_ops}{$method}" . $self->deparse_args(@{$call->{arg}}) . ")[-1]";
                         next;
                     }
 
-                    # Reasign operators, such as: +=, -=, *=, /=, etc...
+                    # Reassignment operators, such as: +=, -=, *=, /=, etc...
                     if (exists $self->{reassign_ops}{$method}) {
+
                         $code =
-                            "do { $code=($code->\${\\'$self->{reassign_ops}{$method}'}"
-                          . $self->deparse_args(@{$call->{arg}})
-                          . "); $code }";
+                          "CORE::sub : lvalue {my \$ref=\\$code; \$\$ref=\$\$ref\->\${\\'$self->{reassign_ops}{$method}'}"
+                          . $self->deparse_args(@{$call->{arg}}) . "}->()";
+
+                        #$code =
+                        #    "do {my \$ref="
+                        #  . ($i == 0 ? '\\' : '')
+                        #  . "$code; \$\$ref=\$\$ref\->\${\\'$self->{reassign_ops}{$method}'}"
+                        #  . (exists($call->{arg}) ? $self->deparse_args(@{$call->{arg}}) : '') . "; "
+                        #  . ($i == $end ? '$' : '')
+                        #  . "\$ref}";
+
                         next;
                     }
 
@@ -1307,36 +1327,44 @@ HEADER
                     if ($method eq '==' or $method eq '!=') {
                         $code =
                             'Sidef::Types::Bool::Bool->new('
-                          . ($method eq '!=' ? 'CORE::not' : '') . '('
-                          . $code . 'eq'
-                          . $self->deparse_args(@{$call->{arg}}) . '))';
+                          . ($method eq '!=' ? 'CORE::not ' : '') . 'do{'
+                          . $code
+                          . '} eq do{'
+                          . $self->deparse_args(@{$call->{arg}}) . '})';
                         next;
                     }
 
                     # <=> method
                     if ($method eq '<=>') {
                         $code =
-                          'Sidef::Types::Number::Number->new(' . $code . 'cmp' . $self->deparse_args(@{$call->{arg}}) . ')';
+                            'Sidef::Types::Number::Number->new(do{'
+                          . $code
+                          . '} cmp do {'
+                          . $self->deparse_args(@{$call->{arg}}) . '})';
                         next;
                     }
 
                     # !~ and ~~ methods
                     if ($method eq '~~' or $method eq '!~') {
                         $self->top_add(qq{use experimental 'smartmatch';\n});
-                        $code = 'Sidef::Types::Bool::Bool->new(' . $code . '~~' . $self->deparse_args(@{$call->{arg}}) . ')';
-                        $code .= '->not' if ($method eq '!~');
+                        $code =
+                            'Sidef::Types::Bool::Bool->new('
+                          . ($method eq '!~' ? 'CORE::not ' : '') . 'do{'
+                          . $code
+                          . '} ~~ do{'
+                          . $self->deparse_args(@{$call->{arg}}) . '})';
                         next;
                     }
 
                     # ! prefix-unary
                     if ($ref eq 'Sidef::Object::Unary') {
                         if ($method eq '!') {
-                            $code = 'Sidef::Types::Bool::Bool->new(!' . $self->deparse_args(@{$call->{arg}}) . ')';
+                            $code = 'Sidef::Types::Bool::Bool->new(!do{' . $self->deparse_args(@{$call->{arg}}) . '})';
                             next;
                         }
 
                         if ($method eq '-') {
-                            $code = $self->deparse_args(@{$call->{arg}}) . '->negate';
+                            $code = $self->deparse_args(@{$call->{arg}}) . '->neg';
                             next;
                         }
 
@@ -1374,11 +1402,21 @@ HEADER
 
                         # Exclamation mark (!) at the end of a method
                         if (substr($method, -1) eq '!') {
-                            $code = '('
-                              . "$old_code=$code->"
+
+                            $code =
+                                "CORE::sub : lvalue {my \$ref=\\$code; \$\$ref=\$\$ref\->"
                               . substr($method, 0, -1)
-                              . (exists($call->{arg}) ? $self->deparse_args(@{$call->{arg}}) : '')
-                              . ", $old_code" . ')[1]';
+                              . (exists($call->{arg}) ? $self->deparse_args(@{$call->{arg}}) : '') . "}->()";
+
+                            #$code =
+                            #    "do {my \$ref="
+                            #  . ($i == 0 ? '\\' : '')
+                            #  . "$code; \$\$ref=\$\$ref\->"
+                            #  . substr($method, 0, -1)
+                            #  . (exists($call->{arg}) ? $self->deparse_args(@{$call->{arg}}) : '') . "; "
+                            #  . ($i == $end ? '$' : '')
+                            #  . "\$ref}";
+
                             next;
                         }
 
@@ -1419,7 +1457,7 @@ HEADER
                 if (exists $call->{block}) {
 
                     # TODO: move above the deparsing of `Block::If`
-                    if ($ref eq 'Sidef::Types::Block::If' and $i == $#{$expr->{call}}) {
+                    if ($ref eq 'Sidef::Types::Block::If' and $i == $end) {
                         $code = "do {\n" . (' ' x $Sidef::SPACES) . $code . $self->deparse_bare_block(@{$call->{block}}) . '}';
                     }
                     else {
