@@ -16,6 +16,7 @@ package Sidef::Deparse::Sidef {
                     class        => 'main',
                     extra_parens => 0,
                     namespaces   => [],
+                    opt          => {},
                     %args,
                    );
         %addr = ();    # reset the addr map
@@ -297,7 +298,7 @@ package Sidef::Deparse::Sidef {
                         $statements[-1] = '(' . $statements[-1] . ')';
                     }
 
-                    if (@statements == 1 and length($statements[0]) <= 80) {
+                    if (@statements == 1 and length($statements[0]) + $Sidef::SPACES <= 80) {
                         $code .= " $statements[0] }";
                     }
                     else {
@@ -463,7 +464,7 @@ package Sidef::Deparse::Sidef {
         elsif ($ref eq 'Sidef::Time::Time') {
             $code = 'Time';
         }
-        elsif ($ref eq 'Sidef::Sys::SIG') {
+        elsif ($ref eq 'Sidef::Sys::Sig') {
             $code = 'Sig';
         }
         elsif ($ref eq 'Sidef::Types::Number::Number') {
@@ -577,20 +578,19 @@ package Sidef::Deparse::Sidef {
             }
         }
 
-        # Indices
+        # Array and hash indices
         if (exists $expr->{ind}) {
             foreach my $ind (@{$expr->{ind}}) {
-                $code .= $self->_dump_array($ind);
-            }
-        }
-
-        if (exists $expr->{lookup}) {
-            foreach my $lookup (@{$expr->{lookup}}) {
-                $code .= '{'
-                  . join(',',
-                         map { ref($_) eq 'HASH' ? ($self->deparse_expr($_)) : $self->deparse_generic('', '', '', $_) }
-                           @{$lookup})
-                  . '}';
+                if (exists $ind->{array}) {
+                    $code .= $self->_dump_array($ind->{array});
+                }
+                else {
+                    $code .= '{'
+                      . join(',',
+                             map { ref($_) eq 'HASH' ? ($self->deparse_expr($_)) : $self->deparse_generic('', '', '', $_) }
+                               @{$ind->{hash}})
+                      . '}';
+                }
             }
         }
 
@@ -655,7 +655,12 @@ package Sidef::Deparse::Sidef {
 
                 if (exists $call->{keyword}) {
                     if ($code ne '') {
-                        $code .= ' ';
+                        if (substr($code, 0, 1) ne '(' or substr($code, -1) ne ')') {
+                            $code .= ' ';
+                        }
+                        else {
+                            $code = "($code)";
+                        }
                     }
                     $code .= $call->{keyword};
                 }
