@@ -75,6 +75,11 @@ package Sidef::Object::Object {
         Sidef::Variable::LazyMethod->new(obj => $self, method => $method, args => \@args);
     }
 
+    #~ sub lazy {
+    #~ my ($self) = @_;
+    #~ Sidef::Lazy::Lazy->new($self);
+    #~ }
+
     sub object_id {
         my ($self) = @_;
         Sidef::Types::Number::Number->new(Scalar::Util::refaddr($self));
@@ -93,6 +98,11 @@ package Sidef::Object::Object {
         Sidef::Types::String::String->new(CORE::ref($obj) || $obj);
     }
 
+    sub bless {
+        my ($obj, $arg) = @_;
+        bless($arg, (CORE::ref($obj) || $obj));
+    }
+
     sub respond_to {
         my ($self, $method) = @_;
         Sidef::Types::Bool::Bool->new($self->can($method));
