@@ -95,13 +95,30 @@ package Sidef::Types::String::String {
     }
 
     sub match {
-        my ($self, $regex, @rest) = @_;
-        $regex->match($self, @rest);
+        my ($self, $regex, $pos) = @_;
+
+        $regex = $regex->to_re
+          if ref($regex) ne 'Sidef::Types::Regex::Regex';
+
+        Sidef::Types::Regex::Match->new(
+                                        obj  => $$self,
+                                        self => $regex,
+                                        pos  => defined($pos) ? $pos->get_value : undef,
+                                       );
     }
 
     sub gmatch {
-        my ($self, $regex, @rest) = @_;
-        $regex->gmatch($self, @rest);
+        my ($self, $regex, $pos) = @_;
+
+        $regex = $regex->to_re
+          if ref($regex) ne 'Sidef::Types::Regex::Regex';
+
+        local $regex->{global} = 1;
+        Sidef::Types::Regex::Match->new(
+                                        obj  => $$self,
+                                        self => $regex,
+                                        pos  => defined($pos) ? $pos->get_value : undef,
+                                       );
     }
 
     sub array_to {
@@ -263,13 +280,13 @@ package Sidef::Types::String::String {
     }
 
     sub first {
-        my ($self) = @_;
-        __PACKAGE__->new(CORE::substr($$self, 0, 1));
+        my ($self, $obj) = @_;
+        __PACKAGE__->new(CORE::substr($$self, 0, defined($obj) ? $obj->get_value : 1));
     }
 
     sub last {
-        my ($self) = @_;
-        __PACKAGE__->new(CORE::substr($$self, -1));
+        my ($self, $obj) = @_;
+        __PACKAGE__->new(CORE::substr($$self, defined($obj) ? -$obj->get_value : -1));
     }
 
     sub char {
